"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RollbarHandler = void 0;
const logger_module_1 = require("../logger.module");
const RollbarHandler = (options) => {
    return (target, methodName, descriptor) => {
        const method = descriptor.value;
        descriptor.value = function () {
            return __awaiter(this, arguments, void 0, function* () {
                try {
                    const result = method.apply(this, arguments);
                    if (result instanceof Promise) {
                        return result.catch((error) => {
                            handleError(target, methodName, error, options);
                        });
                    }
                    return result;
                }
                catch (error) {
                    handleError(target, methodName, error, options);
                }
            });
        };
        return descriptor;
    };
};
exports.RollbarHandler = RollbarHandler;
function handleError(target, methodName, error, options) {
    if (options === null || options === void 0 ? void 0 : options.errorName) {
        error.name = options.errorName;
    }
    logger_module_1.logger.error(error, `${target.constructor.name} -> ${methodName}`);
    if (options === null || options === void 0 ? void 0 : options.rethrow) {
        throw error;
    }
}
